@@ -32,6 +32,24 @@ resource "azurerm_security_center_contact" "bad_example" {
   alert_notifications = true
   alerts_to_admins    = true
 }
+`,
+				`
+resource "azurerm_security_center_contact" "bad_example" {
+	email = ""
+	phone = "+1-555-555-5555"
+  
+	alert_notifications = true
+	alerts_to_admins    = true
+  }
+`,
+				`
+resource "azurerm_security_center_contact" "bad_example" {
+email = ""
+phone = ""
+
+alert_notifications = true
+alerts_to_admins    = true
+}
 `},
 			GoodExample: []string{`
 resource "azurerm_security_center_contact" "good_example" {
@@ -44,6 +62,7 @@ resource "azurerm_security_center_contact" "good_example" {
 `},
 			Links: []string{
 				"https://registry.terraform.io/providers/hashicorp/azurerm/latest/docs/resources/security_center_contact#phone",
+				"https://registry.terraform.io/providers/hashicorp/azurerm/latest/docs/resources/security_center_contact#email",
 				"https://azure.microsoft.com/en-us/services/security-center/",
 			},
 		},
@@ -60,12 +79,34 @@ resource "azurerm_security_center_contact" "good_example" {
 				return
 			}
 
+			if resourceBlock.MissingChild("email") {
+				set.AddResult().
+					WithDescription("Resource '%s' does not have an email address set for the security contact", resourceBlock.FullName())
+
+				return
+			}
+
 			phoneAttr := resourceBlock.GetAttribute("phone")
+			emailAttr := resourceBlock.GetAttribute("email")
+
+			if phoneAttr.IsEmpty() && emailAttr.IsEmpty() {
+				set.AddResult().
+					WithDescription("Resource '%s' does not have a phone number or email set for the security contact", resourceBlock.FullName())
+				return
+			}
+
 			if phoneAttr.IsEmpty() {
 				set.AddResult().
 					WithDescription("Resource '%s' does not have a phone number set for the security contact", resourceBlock.FullName()).
 					WithAttribute(phoneAttr)
 			}
+
+			if emailAttr.IsEmpty() {
+				set.AddResult().
+					WithDescription("Resource '%s' does not have an email address set for the security contact", resourceBlock.FullName()).
+					WithAttribute(emailAttr)
+			}
+
 		},
 	})
 }
